/**
 * 
 */
package uoc.tfc.gien.entities;

import java.util.List;
import java.util.ArrayList;
import java.util.TreeMap;
import java.util.Vector;

import org.json.JSONArray;
import org.json.JSONObject;

import android.content.Context;
import android.util.Log;
import uoc.tfc.gien.database.GieSQLite;
import uoc.tfc.gien.entities.geo.GPSPosition;
import uoc.tfc.gien.entities.geo.Map;
import uoc.tfc.gien.entities.geo.Route;
import uoc.tfc.gien.entities.interactive.InventoryObject;
import uoc.tfc.gien.entities.interactive.Message;
import uoc.tfc.gien.entities.interactive.Option;
import uoc.tfc.gien.entities.interactive.Select;

/**
 * @author Jordi Pérez Sandonís (jordiperez@uoc.edu)
 *
 */
public class Experience {
	
	final String TAG = "Experience";
	
	//data
	private Map map;								// map where the action is happening
	private ArrayList<InventoryObject> inventory;	// inventory objects available for the user
	private TreeMap<String, String> scoreTable;		// points, life, lives, etc
	private ArrayList<Option> actions;				// list of available actions for the user
	
	private GieInfo info;						// information about the experience (author, website, release date, ...)
	
	//ui
	private Boolean mapToolbar;					// indicates if the map toolbar will be shown
	private Boolean compass;					// indicates if the map have compass
	private float zoomLevel;					// indicates the initial zoom level in the map [2.0, 21.0]
	private Boolean actionsVisible;				// indicates if the list of actions will be visible in the UI
	private Boolean inventoryVisible;			// indicates if the inventory will be visible in the UI
	
	//connection
	public enum ConnectionPolicies {			// indicates when gien (the app) will send the position of the user to the server
		ON_GEOEVENT,	// when the user arrives to an active point in the map
		//NEVER,			// never, is a pure offline experience
		REGULARLY;		// every x seconds. Time will be specified in the attribute connectionInterval
	}
	
	// indicates the conditions when the client have to connect with the server
	private boolean connectOnGeoevent;
	private boolean connectRegularly;
	
	private int connectionInterval;				// when policy 'REGULARLY' is present it expresses the server's desired time between
												// connections (in seconds)
	

	/**
	 * Constructor
	 */
	public Experience() {
		this.inventory = new ArrayList<InventoryObject>();
		this.scoreTable = new TreeMap<String, String>(); 
		this.actions = new ArrayList<Option>();
		//this.setConnectionInterval(connectionInterval);
	}

	/**
	 * @return the map
	 */
	public synchronized Map getMap() {
		return map;
	}

	/**
	 * @param maps the maps to set
	 */
	public synchronized void setMap(Map map) {
		this.map = map;
	}

	/**
	 * @return the inventory
	 */
	public synchronized List<InventoryObject> getInventory() {
		return inventory;
	}

	/**
	 * @param inventory the inventory to set
	 */
	public synchronized void setInventory(ArrayList<InventoryObject> inventory) {
		this.inventory = inventory;
	}

	/**
	 * @return the scoreTable
	 */
	public synchronized TreeMap<String, String> getScoreTable() {
		return scoreTable;
	}

	/**
	 * @param scoreTable the scoreTable to set
	 */
	public synchronized void setScoreTable(TreeMap<String, String> scoreTable) {
		this.scoreTable = scoreTable;
	}

	/**
	 * @return the actions
	 */
	public synchronized ArrayList<Option> getActions() {
		return actions;
	}

	/**
	 * @param actions the actions to set
	 */
	public synchronized void setActions(ArrayList<Option> actions) {
		this.actions = actions;
	}
	
	/**
	 * @return the compass
	 */
	public synchronized Boolean isCompassEnabled() {
		return compass;
	}

	/**
	 * @param compass the compass to set
	 */
	public synchronized void setCompass(Boolean compass) {
		this.compass = compass;
	}

	/**
	 * @return the zoomLevel
	 */
	public synchronized float getZoomLevel() {
		return zoomLevel;
	}

	/**
	 * @param zoomLevel the zoomLevel to set
	 */
	public synchronized void setZoomLevel(float zoomLevel) {
		this.zoomLevel = zoomLevel;
	}

	/**
	 * @return the actionsVisible
	 */
	public synchronized Boolean getActionsVisible() {
		return actionsVisible;
	}

	/**
	 * @param actionsVisible the actionsVisible to set
	 */
	public synchronized void setActionsVisible(Boolean actionsVisible) {
		this.actionsVisible = actionsVisible;
	}

	/**
	 * @return the inventoryVisible
	 */
	public synchronized Boolean getInventoryVisible() {
		return inventoryVisible;
	}

	/**
	 * @param inventoryVisible the inventoryVisible to set
	 */
	public synchronized void setInventoryVisible(Boolean inventoryVisible) {
		this.inventoryVisible = inventoryVisible;
	}

	/**
	 * @return the connectionInterval
	 */
	public synchronized int getConnectionInterval() {
		return connectionInterval;
	}

	/**
	 * @param connectionInterval the connectionInterval to set
	 */
	public synchronized void setConnectionInterval(int connectionInterval) {
		this.connectionInterval = connectionInterval;
	}
	
	/**
	 * @return the info
	 */
	public synchronized GieInfo getInfo() {
		return info;
	}

	/**
	 * @param info the info to set
	 */
	public synchronized void setInfo(GieInfo info) {
		this.info = info;
	}
	
	/**
	 * @return the connectOnGeoevent
	 */
	public synchronized boolean isConnectOnGeoevent() {
		return connectOnGeoevent;
	}

	/**
	 * @param connectOnGeoevent the connectOnGeoevent to set
	 */
	public synchronized void setConnectOnGeoevent(boolean connectOnGeoevent) {
		this.connectOnGeoevent = connectOnGeoevent;
	}

	/**
	 * @return the connectRegularly
	 */
	public synchronized boolean isConnectRegularly() {
		return connectRegularly;
	}

	/**
	 * @param connectRegularly the connectRegularly to set
	 */
	public synchronized void setConnectRegularly(boolean connectRegularly) {
		this.connectRegularly = connectRegularly;
	}
	

	/**
	 * @return the mapToolbar
	 */
	public Boolean isMapToolbarEnabled() {
		return mapToolbar;
	}

	/**
	 * @param mapToolbar the mapToolbar to set
	 */
	public void setMapToolbar(Boolean mapToolbar) {
		this.mapToolbar = mapToolbar;
	}

	/**
	 * Loads the object properties from a JSONObject
	 * 
	 * @param object
	 * @param isOnlineExp - indicates if the the experience is online or offline for proper load process
	 * @param context - a context for database access
	 * @return true if success, false if not
	 */
	public synchronized boolean loadJSON(JSONObject object, boolean isOnlineExp, Context context) {
		GieSQLite db = null;
		try {
			//map
			JSONObject jsonMap = object.getJSONObject("map");
			Map m = new Map();
			m.loadJSON(jsonMap);
			this.map = m;
			
			//settings
			JSONObject jsonSettings = object.getJSONObject("settings");
			this.setMapToolbar(jsonSettings.getBoolean("map_toolbar"));
			this.setCompass(jsonSettings.getBoolean("compass"));
			this.setZoomLevel((float)(jsonSettings.getDouble("zoom_level")));
			this.setActionsVisible(jsonSettings.getBoolean("actions_visible"));
			if (isOnlineExp) {
				this.setInventoryVisible(jsonSettings.getBoolean("inventory_visible"));
				JSONArray arr = jsonSettings.getJSONArray("server_connection_policies");
				if (arr.length()>0) {
					for(int i=0; i<arr.length();i++) {
						switch (ConnectionPolicies.valueOf(arr.getString(i))) {
							case ON_GEOEVENT:
								setConnectOnGeoevent(true);
								break;
							case REGULARLY:
								setConnectRegularly(true);
						}
					}
				}
				if (isConnectRegularly()) {
					this.setConnectionInterval(jsonSettings.getInt("server_connection_interval"));
				}
			}
			
			if (!isOnlineExp) {
				
				db = GieSQLite.getInstance(context);
				db.clean(); // clean database to avoid data from past executions
				
				//actions
				JSONArray jsonOptions = object.optJSONArray("action");
				if (jsonOptions.length()>0) {
					for(int i=0; i<jsonOptions.length();i++) {
						Option o = new Option();
						o.loadJSON(jsonOptions.getJSONObject(i));
						this.actions.add(o);
					}
				}
				
				//messages
				ArrayList<Message> messages = new ArrayList<Message>();
				JSONArray jsonMessages = object.optJSONArray("message");
				if (jsonMessages.length()>0) {
					for(int i=0; i<jsonMessages.length();i++) {
						Message msg = new Message();
						msg.loadJSON(jsonMessages.getJSONObject(i));
						messages.add(msg);
					}
					db.insertMessages(messages);
				}
				
				//selects
				ArrayList<Select> selects = new ArrayList<Select>();
				JSONArray jsonSelects = object.optJSONArray("select");
				if (jsonSelects.length()>0) {
					for(int i=0; i<jsonSelects.length();i++) {
						Select s = new Select();
						s.loadJSON(jsonSelects.getJSONObject(i));
						selects.add(s);
					}
					db.insertSelects(selects);
				}
			}
			
		} catch (Exception e) {
			Log.e(TAG,e.getMessage());
			return false;
		} finally {
			/*
			if (db!=null) {
				db.close();
			}
			*/
		}
		return true;
	}
	
	public synchronized boolean loadJSONInventory(JSONObject object) {
		try {
			JSONArray jsonInventory = object.getJSONArray("inventoryObject");
			if (jsonInventory.length()>0) {
				ArrayList<InventoryObject> arr = new ArrayList<InventoryObject>();
				for(int i=0; i<jsonInventory.length();i++) {
					InventoryObject io = new InventoryObject();
					io.loadJSON(jsonInventory.getJSONObject(i));
					arr.add(io);
				}
				this.inventory.clear();
				this.inventory.addAll(arr);
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	
	public synchronized boolean loadJSONScoretable(JSONObject object) {
		try {
			JSONArray jsonScoretable = object.getJSONArray("scoretable");
			if (jsonScoretable.length()>0) {
				TreeMap<String, String> map = new TreeMap<String, String>();
				for(int i=0; i<jsonScoretable.length();i++) {
					PairKV p = new PairKV();
					p.loadJSON(jsonScoretable.getJSONObject(i));
					map.put(p.getKey(), p.getValue());
				}
				this.scoreTable.clear();
				this.scoreTable.putAll(map);
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	
	public synchronized boolean loadJSONOptions(JSONObject object) {
		try {
			JSONArray jsonOptions = object.getJSONArray("option");
			if (jsonOptions.length()>0) {
				ArrayList<Option> arr = new ArrayList<Option>();
				for(int i=0; i<jsonOptions.length();i++) {
					Option o = new Option();
					o.loadJSON(jsonOptions.getJSONObject(i));
					arr.add(o);
				}
				this.actions.clear();
				this.actions.addAll(arr);
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

}
